#!/bin/bash

usage()
{
	echo -e "Usage: $0 <liveburner.ipk> <file 1>...<file n>
\tliveburner.ipk: liveburner file to be updated with ipk
\tfile X: list of ipk and/or scripts to be added to the liveburner"
}

if [ $# -lt 2 ]
then
	usage
	exit 1
fi

UPDATELIVEBURNERLOG=/tmp/updateliveburner.log
exec 3>&1 4>&2 > ${UPDATELIVEBURNERLOG}

LIVEBURNER_PATH=`realpath -L "$1"`
LIVEBURNER_FILENAME=`basename "${LIVEBURNER_PATH}"`
LIVEBURNER_PATH=`dirname "${LIVEBURNER_PATH}"`
WORKING_DIR=`pwd`

# IPK
IPK_ITB_FILENAME="linux.itb"
IPK_CONTROL_GZ_FILENAME="control.tar.gz"
IPK_DATA_GZ_FILENAME="data.tar.gz"
IPK_DEBIANBIN_FILENAME="debian-binary"
IPK_FILES="${IPK_CONTROL_GZ_FILENAME} ${IPK_DATA_GZ_FILENAME} ${IPK_DEBIANBIN_FILENAME}"

# ITB
ITB_KERNEL_FILENAME="zImage"
ITB_DTB_FILENAME="devicetree.dtb"
ITB_INITRDGZ_FILENAME="initrd.cpio.gz"
ITB_ITS_FILENAME="fitimage.its"

# Scripts/Bin
#######
BIN_MKIMAGE="${WORKING_DIR}/mkimage"
BIN_DUMPIMAGE="${WORKING_DIR}/dumpimage"

WORKSPACE="/tmp/updateliveburner"
ITB_WORKSPACE="${WORKSPACE}/ipk/data/tmp"
IPK_WORKSPACE="${WORKSPACE}/ipk"
INITRD_DIR="${ITB_WORKSPACE}/itb"
KEROS_FILENAME="keros.ipk"

INITRD_MAX_SIZE="76000000"
#########################
# IPK Function
#########################

#
# Extract an IPK package
#
# $1: The IPK file to extract
# $2: The workspace to use to extract the content. It allows extraction
#     of multiple elements in the same script without conflicting.
#     e.g: keros, app_X, app_Y...
#
ipk_extract()
{
	IN_IPK="$1"
	IN_WORKSPACE="$2"

	echo "Extracting IPK file \"${IN_IPK}\" in workspace ${IN_WORKSPACE}..."

	ipk_file=`basename "${IN_IPK}"`
	ipk_dir="${IN_WORKSPACE}/ipk"
	ipk_data_dir="${ipk_dir}/data"
	ipk_control_dir="${ipk_dir}/control"

	# IPK directory
	mkdir -p "${ipk_data_dir}" "${ipk_control_dir}"
	cp -f "${IN_IPK}" "${ipk_dir}"

	# Uncompress IPK
	cd ${ipk_dir}
	ar x ${ipk_file}

	# Uncompress data part
	tar -xzf "${IPK_DATA_GZ_FILENAME}" -C "${ipk_data_dir}"
	# Uncompress control part
	tar -xzf "${IPK_CONTROL_GZ_FILENAME}" -C "${ipk_control_dir}"

	rm -f ${IPK_DATA_GZ_FILENAME} ${IPK_CONTROL_GZ_FILENAME} ${ipk_file}
	echo "IPK file \"${IN_IPK}\" extracted"
}

# Build a IPK package from a list of elements:
# - debian-binary
# - data.tar.gz
# - control.tar.gz
#
# $1: The workspace to use as input to generate the IPK
# $2: The result IPK file
ipk_build()
{
	IN_WORKSPACE="$1"
	IN_IPK_FILE="$2"

	echo "Building IPK \"${IN_IPK_FILE}\" from workspace ${IN_WORKSPACE}..."

	ipk_dir="${IN_WORKSPACE}/ipk"
	ipk_data_dir="${ipk_dir}/data"
	ipk_control_dir="${ipk_dir}/control"
	ipk_data_gz_file="${ipk_dir}/${IPK_DATA_GZ_FILENAME}"
	ipk_control_gz_file="${ipk_dir}/${IPK_CONTROL_GZ_FILENAME}"

	cd "${ipk_dir}"
	## DATA part
	#############

	# Recompress data part
	tar --owner=root --group=root -czf "${ipk_data_gz_file}" -C "${ipk_data_dir}" .
	if [ $? -ne 0 ]
	then
		echo "Error when compressing data part"
		exit 1
	fi

	## CONTROL part
	#############

	# Recompress control part
	tar --owner=root --group=root -czf "${ipk_control_gz_file}" -C "${ipk_control_dir}" .
	if [ $? -ne 0 ]
	then
		echo "Error when compressing control part"
		exit 1
	fi

	## IPK Creation
	#############
	ar r "${IN_IPK_FILE}" ${IPK_FILES}
	if [ $? -ne 0 ]
	then
		echo "Error when creating IPK"
		exit 1
	fi

	echo "IPK built: ${IN_IPK_FILE}"
}

#########################
# ITB Function
#########################

#
# Extract a keros ITB file
#
# $1: The ITB file to extract
# $2: The workspace to use to extract the content. It allows extraction
#     of multiple itb in the same script without conflicting.
#     e.g: keros, app_X, app_Y...
#
itb_extract()
{
	IN_ITB_FILE="$1"
	IN_WORKSPACE="$2"

	echo "Extracting Linux ITB \"${IN_ITB_FILE}\" in workspace ${IN_WORKSPACE}..."

	ITB_DIR="${IN_WORKSPACE}/itb"

	# ITB directories
	mkdir -p "${ITB_DIR}"
	ITB_KERNEL_FILE="${ITB_DIR}/${ITB_KERNEL_FILENAME}"
	ITB_DTB_FILE="${ITB_DIR}/${ITB_DTB_FILENAME}"
	ITB_INITRDGZ_FILE="${ITB_DIR}/${ITB_INITRDGZ_FILENAME}"

	# Extracting Kernel
	echo "Extracting Kernel..."

	"${BIN_DUMPIMAGE}" -i "${IN_ITB_FILE}" -T flat_dt -p 0 "${ITB_KERNEL_FILE}"
	if [ $? -ne 0 ]
	then
		echo "Error when extracting kernel from ITB"
		exit 1
	fi
	echo "Kernel extracted"

	# Extracting DTB
	echo "Extracting DTB..."
	"${BIN_DUMPIMAGE}" -i "${IN_ITB_FILE}" -T flat_dt -p 1 "${ITB_DTB_FILE}"
	if [ $? -ne 0 ]
	then
		echo "Error when extracting DTB from ITB"
		exit 1
	fi
	echo "DTB extracted"

	# Extracting INITRD
	echo "Extracting INITRD..."
	"${BIN_DUMPIMAGE}" -i "${IN_ITB_FILE}" -T flat_dt -p 2 "${ITB_INITRDGZ_FILE}"
	if [ $? -ne 0 ]
	then
		echo "Error when extracting INITRD from ITB"
		exit 1
	fi
	echo "INITRD extracted"

	echo "Linux ITB extracted"
}


#
# Build a Linux ITB from a dir with 3 elements:
# - DTB: devicetree.dtb
# - INITRD: initrd.cpio.gz
# - KERNEL: zImage
#
# $1: The workspace to use as input dir.
# $2: The final initrd file with cpio.gz extension
#
itb_build()
{
	IN_WORKSPACE="$1"
	IN_ITB_FILE="$2"

	echo "Building Linux ITB from workspace ${IN_WORKSPACE}..."

	itb_dir="${IN_WORKSPACE}/itb"

	cd "${itb_dir}"
	rm -f "${IN_ITB_FILE}"
	cp -f "${WORKING_DIR}/${ITB_ITS_FILENAME}" "${itb_dir}"
	"${BIN_MKIMAGE}" -A arm -f "${ITB_ITS_FILENAME}" "${IN_ITB_FILE}" 2>&1
	if [ $? -ne 0 ]
	then
		echo "Error when creating Linux ITB"
		exit 1
	fi

	rm -rf "${ITB_WORKSPACE}/itb"
	echo "Linux ITB created: ${IN_ITB_FILE}"
}

#########################
# INITRD functions
#########################

#
# Extract a cpio.gz INITRD
#
# $1: The INITRD in cpio.gz format file to extract
# $2: The workspace to use to extract the content. It allows extraction
#     of multiple elements in the same script without conflicting.
#     e.g: keros, app_X, app_Y...
#
initrd_extract()
{
	IN_INITRD_FILE="$1"
	IN_WORKSPACE="$2"

	echo "Extracting INITRD \"${IN_INITRD_FILE}\" in workspace ${IN_WORKSPACE}..."

	# INITRD dir
	initrd_dir="${IN_WORKSPACE}/initrd"
	mkdir -p "${initrd_dir}"

	cd "${initrd_dir}"
	gzip -dc "${IN_INITRD_FILE}" | cpio -idm 2>&1
	if [ $? -ne 0 ]
	then
		echo "Error when extracting INITRD"
		exit 1
	fi
	echo "INITRD extracted"

}

#
# Build an INITRD from a dir
#
# $1: The workspace to use as input dir.
# $1: The final initrd file with cpio.gz extension
#
initrd_build()
{
	IN_WORKSPACE="$1"
	IN_INITRD_FILE="$2"

	echo "Building INITRD from workspace ${IN_WORKSPACE}..."

	initrd_dir="${IN_WORKSPACE}/initrd"

	cd "${initrd_dir}"
	rm -f "${IN_INITRD_FILE}"
	find . | cpio --create --format='newc' --quiet | gzip > "${IN_INITRD_FILE}"
	rm -rf "${INITRD_DIR}/initrd"
	echo "INITRD created: ${IN_INITRD_FILE}"

}

#########################
# MAIN
#########################

ipk_extract "$1" "${WORKSPACE}"
itb_extract "${ITB_WORKSPACE}/linux.itb" "${ITB_WORKSPACE}"
initrd_extract "${INITRD_DIR}/initrd.cpio.gz" "${INITRD_DIR}"

INITRD_SIZE=`du -sb ${INITRD_DIR}/initrd | sed 's/\s.*$//'`
CUSTOM_IPK_MAX_SIZE=$((INITRD_MAX_SIZE - INITRD_SIZE))
CUSTOM_IPK_SIZE=0

cd "${WORKING_DIR}"
shift
while [ $# != 0 ]
do
	if [ -f "$1" ]
	then
		FILESIZE=`stat -c%s "$1"`
		CUSTOM_IPK_SIZE=$((CUSTOM_IPK_SIZE + FILESIZE))
		FILENAME=$(basename -- "$1")
		if [ ${CUSTOM_IPK_SIZE} -gt ${CUSTOM_IPK_MAX_SIZE} ]
		then
			echo "Error : custom IPK size exceed max size (${CUSTOM_IPK_MAX_SIZE} bytes) with file $1"
			rm -rf ${WORKSPACE}
			exit 1
		fi

		# preinst script is specific and must be installed in liveburner
		if [ "${FILENAME}" = "preinst" ]
		then
			cp "$1" "${IPK_WORKSPACE}/control/"
			chmod 777 "${IPK_WORKSPACE}/control/${FILENAME}"
		else
			cp "$1" "${INITRD_DIR}/initrd/burner/"
			chmod 777 "${INITRD_DIR}/initrd/burner/${FILENAME}"
		fi
		shift
	else
		echo "Error : File $1 not found"
		rm -rf ${WORKSPACE}
		exit 1
	fi
done

initrd_build "${INITRD_DIR}" "${INITRD_DIR}/initrd.cpio.gz"
itb_build "${ITB_WORKSPACE}" "${ITB_WORKSPACE}/linux.itb"
ipk_build "${WORKSPACE}" "${LIVEBURNER_PATH}/custom-${LIVEBURNER_FILENAME}"

rm -rf "${WORKSPACE}"

exec echo "Liveburner custom-${LIVEBURNER_FILENAME} created" 1>&3
